@@ -7,7 +7,7 @@
  *      implementation of the pqxx::connection_base abstract base class.
  *   pqxx::connection_base encapsulates a frontend to backend connection
  *
- * Copyright (c) 2001-2004, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2005, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -32,6 +32,8 @@
 #endif	// PQXX_HAVE_SYS_SELECT_H
 
 #include "pqxx/connection_base"
+#include "pqxx/nontransaction"
+#include "pqxx/pipeline"
 #include "pqxx/result"
 #include "pqxx/transaction"
 #include "pqxx/trigger"
@@ -77,7 +79,7 @@ pqxx::connection_base::connection_base(const char ConnInfo[]) :
 }
 
 
-void pqxx::connection_base::Connect()
+void pqxx::connection_base::activate()
 {
   if (!is_open())
   {
@@ -109,17 +111,6 @@ void pqxx::connection_base::deactivate()
 }
 
 
-/// Initiate a connection, to at least the point where we have a PQconn
-void pqxx::connection_base::halfconnect()
-{
-  if (!m_Conn)
-  {
-    startconnect();
-    if (!m_Conn) Connect();
-  }
-}
-
-
 void pqxx::connection_base::set_variable(const string &Var, const string &Value)
 {
   if (m_Trans.get())
@@ -159,7 +150,7 @@ string pqxx::connection_base::RawGetVar(const string &Var)
  */
 void pqxx::connection_base::SetupState()
 {
-  if (!m_Conn) 
+  if (!m_Conn)
     throw logic_error("libpqxx internal error: SetupState() on no connection");
 
   if (Status() != CONNECTION_OK)
@@ -175,30 +166,44 @@ void pqxx::connection_base::SetupState()
 
   InternalSetTrace();
 
-  // Reinstate all active triggers
-  if (!m_Triggers.empty())
+  if (!m_Triggers.empty() || !m_Vars.empty())
   {
-    const TriggerList::const_iterator End = m_Triggers.end();
-    string Last;
-    for (TriggerList::const_iterator i = m_Triggers.begin(); i != End; ++i)
+    // Pipeline all queries needed to restore triggers and variables.  Use
+    // existing nontransaction if we're in one, or set up a temporary one.
+    // Note that at this stage, the presence of a transaction object doesn't
+    // mean much.  Even if it is a dbtransaction, no BEGIN has been issued yet
+    // in the new connection.
+    auto_ptr<nontransaction> n;
+    if (!m_Trans.get())
+      n=auto_ptr<nontransaction>(new nontransaction(*this, "connection_setup"));
+    pipeline p(*m_Trans.get(), "restore_state");
+    p.retain(m_Triggers.size() + m_Vars.size());
+
+    // Reinstate all active triggers
+    if (!m_Triggers.empty())
     {
-      // m_Triggers can handle multiple Triggers waiting on the same event; 
-      // issue just one LISTEN for each event.
-      if (i->first != Last)
+      const TriggerList::const_iterator End = m_Triggers.end();
+      string Last;
+      for (TriggerList::const_iterator i = m_Triggers.begin(); i != End; ++i)
       {
-	// TODO: Concatenate these
-	const string LQ("LISTEN \"" + i->first + "\"");
-        result R( PQexec(m_Conn, LQ.c_str()) );
-        R.CheckStatus(LQ);
-        Last = i->first;
+        // m_Triggers can handle multiple Triggers waiting on the same event;
+        // issue just one LISTEN for each event.
+        if (i->first != Last)
+        {
+	  p.insert("LISTEN \"" + i->first + "\"");
+          Last = i->first;
+        }
       }
     }
-  }
 
-  for (map<string,string>::const_iterator i=m_Vars.begin(); 
-       i!=m_Vars.end(); 
-       ++i)
-    RawSetVar(i->first, i->second);
+    for (map<string,string>::const_iterator i=m_Vars.begin();
+         i!=m_Vars.end();
+         ++i)
+      p.insert("SET " + i->first + "=" + i->second);
+
+    // Now do the whole batch at once
+    while (!p.empty()) p.retrieve();
+  }
 }
 
 
@@ -218,7 +223,7 @@ bool pqxx::connection_base::is_open() const throw ()
 }
 
 
-auto_ptr<pqxx::noticer> 
+auto_ptr<pqxx::noticer>
 pqxx::connection_base::set_noticer(auto_ptr<noticer> N) throw ()
 {
   if (m_Conn)
@@ -226,7 +231,7 @@ pqxx::connection_base::set_noticer(auto_ptr<noticer> N) throw ()
     if (N.get()) PQsetNoticeProcessor(m_Conn, pqxxNoticeCaller, N.get());
     else PQsetNoticeProcessor(m_Conn, 0, 0);
   }
-  
+
   auto_ptr<noticer> Old = m_Noticer;
   m_Noticer = N;
 
@@ -247,7 +252,7 @@ void pqxx::connection_base::process_notice_raw(const char msg[]) throw ()
 
 void pqxx::connection_base::process_notice(const char msg[]) throw ()
 {
-  if (!msg) 
+  if (!msg)
   {
     process_notice_raw("NULL pointer in client program message!\n");
   }
@@ -256,7 +261,7 @@ void pqxx::connection_base::process_notice(const char msg[]) throw ()
     const size_t len = strlen(msg);
     if (len > 0)
     {
-      if (msg[len-1] == '\n') 
+      if (msg[len-1] == '\n')
       {
 	process_notice_raw(msg);
       }
@@ -331,7 +336,7 @@ void pqxx::connection_base::AddTrigger(pqxx::trigger *T)
   if (m_Conn && (p == m_Triggers.end()))
   {
     // Not listening on this event yet, start doing so.
-    const string LQ("LISTEN \"" + T->name() + "\""); 
+    const string LQ("LISTEN \"" + T->name() + "\"");
     result R( PQexec(m_Conn, LQ.c_str()) );
 
     try
@@ -370,10 +375,10 @@ void pqxx::connection_base::RemoveTrigger(pqxx::trigger *T) throw ()
 
     const TriggerList::iterator i = find(R.first, R.second, E);
 
-    if (i == R.second) 
+    if (i == R.second)
     {
-      process_notice("Attempt to remove unknown trigger '" + 
-		     E.first + 
+      process_notice("Attempt to remove unknown trigger '" +
+		     E.first +
 		     "'");
     }
     else
@@ -393,7 +398,7 @@ void pqxx::connection_base::RemoveTrigger(pqxx::trigger *T) throw ()
 
 int pqxx::connection_base::get_notifs()
 {
-  int notifs = 0; 
+  int notifs = 0;
   if (!is_open()) return notifs;
 
   PQconsumeInput(m_Conn);
@@ -418,8 +423,8 @@ int pqxx::connection_base::get_notifs()
       try
       {
 	process_notice("Exception in trigger handler '" +
-		       i->first + 
-		       "': " + 
+		       i->first +
+		       "': " +
 		       e.what() +
 		       "\n");
       }
@@ -486,7 +491,7 @@ pqxx::result pqxx::connection_base::exec_prepared(const char QueryName[],
   {
     Retries--;
     Reset();
-    if (is_open()) 
+    if (is_open())
       R = PQexecPrepared(m_Conn, QueryName, NumParams, Params, NULL, NULL, 0);
   }
 
@@ -500,7 +505,7 @@ pqxx::result pqxx::connection_base::exec_prepared(const char QueryName[],
   return R;
 #else
   stringstream Q;
-  Q << "EXECUTE " 
+  Q << "EXECUTE "
     << QueryName
     << ' '
     << separated_list(",", Params, Params+NumParams);
@@ -516,27 +521,31 @@ void pqxx::connection_base::Reset()
   // Forget about any previously ongoing connection attempts
   dropconnect();
 
-  // Attempt to set up or restore connection
-  if (!m_Conn) 
-  {
-    Connect();
-  }
-  else 
+  if (m_Conn)
   {
+    // Reset existing connection
     PQreset(m_Conn);
     SetupState();
     clear_fdmask();
   }
+  else
+  {
+    // No existing connection--start a new one
+    activate();
+  }
 }
 
 
 void pqxx::connection_base::close() throw ()
 {
+#ifdef PQXX_QUIET_DESTRUCTORS
+  set_noticer(PGSTD::auto_ptr<noticer>(new nonnoticer()));
+#endif
   clear_fdmask();
   try
   {
-    if (m_Trans.get()) 
-      process_notice("Closing connection while " + 
+    if (m_Trans.get())
+      process_notice("Closing connection while " +
 	             m_Trans.get()->description() + " still open");
 
     if (!m_Triggers.empty())
@@ -584,7 +593,7 @@ void pqxx::connection_base::RegisterTransaction(transaction_base *T)
 }
 
 
-void pqxx::connection_base::UnregisterTransaction(transaction_base *T) 
+void pqxx::connection_base::UnregisterTransaction(transaction_base *T)
   throw ()
 {
   try
@@ -600,7 +609,7 @@ void pqxx::connection_base::UnregisterTransaction(transaction_base *T)
 
 void pqxx::connection_base::MakeEmpty(pqxx::result &R)
 {
-  if (!m_Conn) 
+  if (!m_Conn)
     throw logic_error("libpqxx internal error: MakeEmpty() on null connection");
 
   R = result(PQmakeEmptyPGresult(m_Conn, PGRES_EMPTY_QUERY));
@@ -677,7 +686,7 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
 
   Result = (Line != theWriteTerminator);
 
-  if (Result) 
+  if (Result)
   {
     if (!Line.empty() && (Line[Line.size()-1] == '\n'))
       Line.erase(Line.size()-1);
@@ -745,7 +754,7 @@ void pqxx::connection_base::EndCopyWrite()
   // This check is a little odd, but for some reason PostgreSQL 7.4 keeps
   // returning 1 (i.e., failure) but with an empty error message, and without
   // anything seeming wrong.
-  if ((PQendcopy(m_Conn) != 0) && ErrMsg() && *ErrMsg()) 
+  if ((PQendcopy(m_Conn) != 0) && ErrMsg() && *ErrMsg())
     throw runtime_error(ErrMsg());
 #endif
 }
