@@ -25,6 +25,7 @@
 #include "pqxx/largeobject"
 
 using namespace PGSTD;
+using namespace pqxx::internal;
 using namespace pqxx::internal::pq;
 
 namespace
@@ -107,7 +108,7 @@ void pqxx::largeobject::to_file(dbtransaction &T, const string &File) const
 void pqxx::largeobject::remove(dbtransaction &T) const
 {
   if (lo_unlink(RawConnection(T), id()) == -1)
-    throw runtime_error("Could not delete large object " + 
+    throw runtime_error("Could not delete large object " +
 	                to_string(m_ID) + ": " +
 			Reason());
 }
@@ -161,12 +162,12 @@ pqxx::largeobjectaccess::largeobjectaccess(dbtransaction &T,
 }
 
 
-pqxx::largeobjectaccess::size_type 
+pqxx::largeobjectaccess::size_type
 pqxx::largeobjectaccess::seek(size_type dest, seekdir dir)
 {
   const size_type Result = cseek(dest, dir);
   if (Result == -1)
-    throw runtime_error("Error seeking in large object: " + Reason()); 
+    throw runtime_error("Error seeking in large object: " + Reason());
 
   return Result;
 }
@@ -201,7 +202,7 @@ void pqxx::largeobjectaccess::write(const char Buf[], size_type Len)
                           "#" + to_string(id()) + ": " +
 	                  Reason());
     if (Bytes == 0)
-      throw runtime_error("Could not write to large object #" + 
+      throw runtime_error("Could not write to large object #" +
 	                  to_string(id()) + ": " + Reason());
 
     throw runtime_error("Wanted to write " + to_string(Len) + " bytes "
@@ -211,7 +212,7 @@ void pqxx::largeobjectaccess::write(const char Buf[], size_type Len)
 }
 
 
-pqxx::largeobjectaccess::size_type 
+pqxx::largeobjectaccess::size_type
 pqxx::largeobjectaccess::read(char Buf[], size_type Len)
 {
   const long Bytes = cread(Buf, Len);
@@ -233,6 +234,9 @@ void pqxx::largeobjectaccess::open(openmode mode)
 
 void pqxx::largeobjectaccess::close() throw ()
 {
+#ifdef PQXX_QUIET_DESTRUCTORS
+  disable_noticer Quiet(m_Trans.conn());
+#endif
   if (m_fd >= 0) lo_close(RawConnection(), m_fd);
 }
 
