@@ -7,7 +7,7 @@
  *      implementation of the pqxx::tablewriter class.
  *   pqxx::tablewriter enables optimized batch updates to a database table
  *
- * Copyright (c) 2001-2004, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2005, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -24,7 +24,7 @@
 using namespace PGSTD;
 
 
-pqxx::tablewriter::tablewriter(transaction_base &T, 
+pqxx::tablewriter::tablewriter(transaction_base &T,
     const string &WName,
     const string &Null) :
   tablestream(T, WName, Null, "tablewriter")
@@ -35,6 +35,9 @@ pqxx::tablewriter::tablewriter(transaction_base &T,
 
 pqxx::tablewriter::~tablewriter() throw ()
 {
+#ifdef PQXX_QUIET_DESTRUCTORS
+  internal::disable_noticer Quiet(m_Trans.conn());
+#endif
   try
   {
     writer_close();
@@ -86,11 +89,11 @@ void pqxx::tablewriter::writer_close()
   if (!is_finished())
   {
     base_close();
-    try 
-    { 
-      m_Trans.EndCopyWrite(); 
-    } 
-    catch (const exception &) 
+    try
+    {
+      m_Trans.EndCopyWrite();
+    }
+    catch (const exception &)
     {
       try { base_close(); } catch (const exception &) {}
       throw;
